import os
from .template import Template


class TemplateWriter(object):

    def __init__(self, eureka_client, template_file, output_file):
        self._output_file = output_file
        self._template = Template(eureka_client, template_file)
        self.rendered = None

    def render(self):
        self.rendered = self._template.render()
        return self.rendered

    def write(self):
        if not os.path.exists(self._output_file) or \
           self.rendered != self.render():

            self._write_file(self.render())
            return True

        return False

    def _write_file(self, content):
        with open(self._output_file, 'w') as f:
            f.write(content)



from .application import ApplicationCollection
from jinja2 import Environment, FileSystemLoader
import os


class Template(object):
    def __init__(self, eureka_client, template_file):
        self._eureka_client = eureka_client
        self._template_file = os.path.abspath(template_file)

    def _get_template(self):
        env = Environment(loader=FileSystemLoader('/'))
        return env.get_template(self._template_file)

    def _get_apps(self):
        apps_info = self._eureka_client.get_apps()["applications"]
        return ApplicationCollection.from_info(apps_info["application"])

    def render(self):
        return self._get_template().render(apps=self._get_apps())



import subprocess


class Command(object):
    def __init__(self, command):
        self._command = command

    def execute(self):
        if self._command:
            subprocess.call(self._command, shell=True)



from setuptools import setup

setup(
    name='eureka_template',
    version='0.0.4',
    description='Eureka Templating Library',
    author='Jorge Dias',
    author_email='jorge@mrdias.com',
    packages=['eureka_template'],
    scripts=['bin/eureka-template'],
    install_requires=[
        'eureka-client>=0.6.2',
        'jinja2'
    ]
)



class Instance(object):

    def __init__(self, eureka_information):
        self._eureka_information = eureka_information

    def get(self, value_path):
        parts = value_path.split(".")
        info = self._eureka_information

        for part in parts:
            if not isinstance(info, dict):
                raise KeyError
            result = info[part]
            info = result

        return result

    @property
    def port(self):
        return self.get("port.$")

    @property
    def port_enabled(self):
        return self.get("port.@enabled") == "true"

    @property
    def secure_port(self):
        return self.get("securePort.$")

    @property
    def secure_port_enabled(self):
        return self.get("securePort.@enabled") == "true"

    @property
    def vip_address(self):
        return self.get("vipAddress")

    @property
    def secure_vip_address(self):
        return self.get("secureVipAddress")

    @property
    def ip_address(self):
        return self.get("ipAddr")

    @property
    def hostname(self):
        return self.get("hostName")


class Application(object):

    @classmethod
    def from_info(cls, eureka_information):
        name = eureka_information["name"]

        instances_info = eureka_information["instance"]

        if not isinstance(instances_info, list):
            instances_info = [instances_info]

        instances = [Instance(info) for info in instances_info]
        return cls(name, instances)

    def __init__(self, name, instances):
        self.name = name
        self.instances = instances


class ApplicationCollection(object):

    @classmethod
    def from_info(cls, eureka_information):
        applications = {}

        for app_info in eureka_information:
            app = Application.from_info(app_info)
            applications[app.name] = app

        return cls(applications)

    def __init__(self, applications):
        self._applications = applications

    def __iter__(self):
        return self._applications.itervalues()

    def __contains__(self, name):
        return name in self._applications

    def __len__(self):
        return len(self._applications)

    def get(self, name):
        return self._applications.get(name)
